@@ -32,6 +32,7 @@ BitmapImage::BitmapImage(const BitmapImage& a) : KeyFrame(a)
 {
     mBounds = a.mBounds;
     mMinBound = a.mMinBound;
+    mEnableAutoCrop = a.mEnableAutoCrop;
     mImage = std::make_shared<QImage>(*a.mImage);
 }
 
@@ -433,8 +434,8 @@ void BitmapImage::autoCrop()
     int relRight = mBounds.width()-1;
 
     // Check left row
-    isEmpty = true; // Reset isEmpty
-    while (isEmpty && relLeft <= relRight) // Loop through columns
+    isEmpty = (relBottom >= relTop); // Check left only when 
+    while (isEmpty && relBottom >= relTop && relLeft <= relRight) // Loop through columns
     {
         // Point cursor to the pixel at row relTop and column relLeft
         const QRgb* cursor = reinterpret_cast<const QRgb*>(mImage->constScanLine(relTop)) + relLeft;
@@ -467,7 +468,7 @@ void BitmapImage::autoCrop()
     }
 
     // Check right row
-    isEmpty = true; // Reset isEmpty
+    isEmpty = (relBottom >= relTop); // Reset isEmpty
     while (isEmpty && relRight >= relLeft) // Loop through columns
     {
         // Point cursor to the pixel at row relTop and column relRight
