@@ -44,6 +44,9 @@ enum class SETTING
     AUTO_SAVE,
     AUTO_SAVE_NUMBER,
     SHORT_SCRUB,
+    FPS,
+    FIELD_W,
+    FIELD_H,
     FRAME_SIZE,
     TIMELINE_SIZE,
     LABEL_FONT_SIZE,
@@ -54,12 +57,17 @@ enum class SETTING
     ONION_NEXT_FRAMES_NUM,
     ONION_WHILE_PLAYBACK,
     ONION_TYPE,
-    GRID_SIZE,
+    FLIP_ROLL_MSEC,
+    FLIP_ROLL_DRAWINGS,
+    FLIP_INBETWEEN_MSEC,
+    GRID_SIZE_W,
+    GRID_SIZE_H,
     QUICK_SIZING,
     MULTILAYER_ONION,
     LANGUAGE,
     LAYOUT_LOCK,
     DRAW_ON_EMPTY_FRAME_ACTION,
+    FRAME_POOL_SIZE,
     COUNT, // COUNT must always be the last one.
 };
 
