@@ -24,80 +24,14 @@ GNU General Public License for more details.
 #include <QProcess>
 #include <QApplication>
 #include <QStandardPaths>
+#include <QThread>
+#include <QtMath>
+
 #include "object.h"
 #include "layercamera.h"
 #include "layersound.h"
 #include "soundclip.h"
 
-// refs
-// http://www.topherlee.com/software/pcm-tut-wavformat.html
-// http://soundfile.sapp.org/doc/WaveFormat/
-//
-struct WavFileHeader
-{
-    char    riff[4];
-    int32_t chuckSize;
-    char    format[4];
-    char    fmtID[4];
-    int32_t fmtChuckSize;
-    int16_t audioFormat;
-    int16_t numChannels;
-    int32_t sampleRate;
-    int32_t byteRate;
-    int16_t blockAlign;
-    int16_t bitsPerSample;
-    char    dataChuckID[4];
-    int32_t dataSize;
-
-    void InitWithDefaultValues()
-    {
-        strncpy(riff, "RIFF", 4);
-        chuckSize = 0;
-        strncpy(format, "WAVE", 4);
-        strncpy(fmtID, "fmt ", 4);
-        fmtChuckSize = 16;
-        audioFormat = 1; // 1 means PCM
-        numChannels = 2; // stereo
-        sampleRate = 44100;
-        bitsPerSample = 16;
-        blockAlign = (bitsPerSample * numChannels) / 8;
-        byteRate = (sampleRate * bitsPerSample * numChannels) / 8;
-
-        strncpy(dataChuckID, "data", 4);
-        dataSize = 0;
-    }
-};
-
-int16_t safeSumInt16(int16_t a, int16_t b)
-{
-    int32_t a32 = static_cast<int32_t>(a);
-    int32_t b32 = static_cast<int32_t>(b);
-
-    if ((a32 + b32) > INT16_MAX)
-    {
-        return INT16_MAX;
-    }
-    else if ((a32 + b32) < INT16_MIN)
-    {
-        return INT16_MIN;
-    }
-    return a + b;
-}
-
-void skipUselessChucks(WavFileHeader& header, QFile& file)
-{
-    // We only care about the 'data' chuck
-    while (memcmp(header.dataChuckID, "data", 4) != 0)
-    {
-        int skipByteCount = header.dataSize;
-        std::vector<char> skipData(skipByteCount);
-        file.read(skipData.data(), skipByteCount);
-
-        file.read((char*)&header.dataChuckID, 4);
-        file.read((char*)&header.dataSize, 4);
-    }
-}
-
 QString ffmpegLocation()
 {
 #ifdef _WIN32
@@ -161,7 +95,7 @@ Status MovieExporter::run(const Object* obj,
 {
     majorProgress(0.f, 0.03f);
     minorProgress(0.f);
-    progressMessage(QApplication::tr("Checking environment..."));
+    progressMessage(QObject::tr("Checking environment..."));
 
     clock_t t1 = clock();
 
@@ -171,8 +105,6 @@ Status MovieExporter::run(const Object* obj,
     {
 #ifdef _WIN32
         qCritical() << "Please place ffmpeg.exe in " << ffmpegPath << " directory";
-#elif __APPLE__
-        qCritical() << "Please place ffmpeg in " << ffmpegPath << " directory";
 #else
         qCritical() << "Please place ffmpeg in " << ffmpegPath << " directory";
 #endif
@@ -214,7 +146,7 @@ Status MovieExporter::run(const Object* obj,
     }
     minorProgress(1.f);
     majorProgress(1.f, 1.f);
-    progressMessage(QApplication::tr("Done"));
+    progressMessage(QObject::tr("Done"));
     
     clock_t t2 = clock() - t1;
     qDebug("MOVIE = %.1f sec", static_cast<float>(t2) / CLOCKS_PER_SEC);
@@ -244,22 +176,13 @@ Status MovieExporter::assembleAudio(const Object* obj,
                                     std::function<void(float)> progress)
 {
     // Quicktime assemble call
-    int startFrame = mDesc.startFrame;
-    int endFrame = mDesc.endFrame;
-    int fps = mDesc.fps;
+    const int startFrame = mDesc.startFrame;
+    const int endFrame = mDesc.endFrame;
+    const int fps = mDesc.fps;
 
     Q_ASSERT(startFrame >= 0);
     Q_ASSERT(endFrame >= startFrame);
 
-    float lengthInSec = (endFrame - startFrame + 1) / (float)fps;
-    qDebug() << "Audio Length = " << lengthInSec << " seconds";
-
-    int32_t audioDataSize = 44100 * 2 * 2 * lengthInSec;
-
-    std::vector<int16_t> audioData(audioDataSize / sizeof(int16_t));
-
-    bool audioDataValid = false;
-
     QDir dir(mTempWorkDir);
     Q_ASSERT(dir.exists());
 
@@ -279,76 +202,46 @@ Status MovieExporter::assembleAudio(const Object* obj,
 
     int clipCount = 0;
 
-    for (SoundClip* clip : allSoundClips)
+    QString strCmd, filterComplex, amergeInput, panChannelLayout;
+    strCmd += QString("\"%1\"").arg(ffmpegPath);
+
+    int wholeLen = qCeil((endFrame - startFrame) * 44100.0 / fps);
+    for (auto clip : allSoundClips)
     {
         if (mCanceled)
         {
             return Status::CANCELED;
         }
 
-        // convert audio file: 44100Hz sampling rate, stereo, signed 16 bit little endian
-        // supported audio file types: wav, mp3, ogg... ( all file types supported by ffmpeg )
-        QString strCmd;
-        strCmd += QString("\"%1\"").arg(ffmpegPath);
-        strCmd += QString(" -i \"%1\" ").arg(clip->fileName());
-        strCmd += "-ar 44100 -acodec pcm_s16le -ac 2 -y ";
-        strCmd += QString("\"%1\"").arg(tempAudioPath);
+        // Add sound file as input
+        strCmd += QString(" -i \"%1\"").arg(clip->fileName());
 
-        executeFFMpeg(strCmd, [](float){});
-        qDebug() << "audio file: " + tempAudioPath;
+        // Offset the sound to its correct position
+        // See https://superuser.com/questions/716320/ffmpeg-placing-audio-at-specific-location
+        filterComplex += QString("[%1:a:0] aformat=sample_fmts=fltp:sample_rates=44100:channel_layouts=mono,volume=1,adelay=%2S|%2S,apad=whole_len=%3[ad%1];")
+                    .arg(clipCount).arg(qRound(44100.0 * (clip->pos() - 1) / fps)).arg(wholeLen);
+        amergeInput += QString("[ad%1]").arg(clipCount);
+        panChannelLayout += QString("c%1+").arg(clipCount);
 
-        // Read wav file header
-        WavFileHeader header;
-        QFile file(tempAudioPath);
-        file.open(QIODevice::ReadOnly);
-        file.read((char*)&header, sizeof(WavFileHeader));
-
-        skipUselessChucks(header, file);
-
-        int32_t audioSize = header.dataSize;
-
-        qDebug() << "audio len " << audioSize;
-
-        // before calling malloc should check: audioSize < max credible value
-        std::vector< int16_t > data(audioSize / sizeof(int16_t));
-        file.read((char*)data.data(), audioSize);
-        audioDataValid = true;
-
-        float fframe = (float)clip->pos() / (float)fps;
-        int delta = fframe * 44100 * 2;
-        qDebug() << "audio delta " << delta;
-
-        int indexMax = std::min(audioSize / 2, audioDataSize / 2 - delta);
-
-        // audio files 'mixing': 'higher' sound layers overwrite 'lower' sound layers
-        for (int i = 0; i < indexMax; i++)
-        {
-            audioData[i + delta] = safeSumInt16(audioData[i + delta], data[i]);
-        }
-
-        file.close();
-
-        progress((float)clipCount / allSoundClips.size());
         clipCount++;
     }
+    // Remove final '+'
+    panChannelLayout.chop(1);
+    // Output arguments
+    // Mix audio
+    strCmd += QString(" -filter_complex \"%1%2 amerge=inputs=%3, pan=mono|c0=%4 [out]\"")
+            .arg(filterComplex).arg(amergeInput).arg(clipCount).arg(panChannelLayout);
+    // Convert audio file: 44100Hz sampling rate, stereo, signed 16 bit little endian
+    // Supported audio file types: wav, mp3, ogg... ( all file types supported by ffmpeg )
+    strCmd += " -ar 44100 -acodec pcm_s16le -ac 2 -map \"[out]\" -y";
+    // Trim audio
+    strCmd += QString(" -ss %1").arg((startFrame - 1) / static_cast<double>(fps));
+    strCmd += QString(" -to %1").arg(endFrame / static_cast<double>(fps));
+    // Output path
+    strCmd += " " + mTempWorkDir + "/tmpaudio.wav";
 
-    if (!audioDataValid)
-    {
-        return Status::SAFE;
-    }
-
-    // save mixed audio file ( will be used as audio stream )
-    QFile file(mTempWorkDir + "/tmpaudio.wav");
-    file.open(QIODevice::WriteOnly);
-
-    WavFileHeader outputHeader;
-    outputHeader.InitWithDefaultValues();
-    outputHeader.dataSize = audioDataSize;
-    outputHeader.chuckSize = 36 + audioDataSize;
-
-    file.write((char*)&outputHeader, sizeof(outputHeader));
-    file.write((char*)audioData.data(), audioDataSize);
-    file.close();
+    STATUS_CHECK(executeFFMpeg(strCmd, progress));
+    qDebug() << "audio file: " + tempAudioPath;
 
     return Status::OK;
 }
@@ -385,7 +278,7 @@ Status MovieExporter::generateMovie(
     int frameStart = mDesc.startFrame;
     int frameEnd = mDesc.endFrame;
     const QSize exportSize = mDesc.exportSize;
-    bool transparency = false;
+    bool transparency = mDesc.alpha;
     QString strCameraName = mDesc.strCameraName;
     bool loop = mDesc.loop;
 
@@ -409,8 +302,6 @@ Status MovieExporter::generateMovie(
     }
     imageToExportBase.fill(bgColor);
 
-    QTransform view = cameraLayer->getViewAtFrame(currentFrame);
-
     QSize camSize = cameraLayer->getViewSize();
     QTransform centralizeCamera;
     centralizeCamera.translate(camSize.width() / 2, camSize.height() / 2);
@@ -432,7 +323,7 @@ Status MovieExporter::generateMovie(
 
     QString strCmd = QString("\"%1\"").arg(ffmpegPath);
     strCmd += QString(" -f rawvideo -pixel_format bgra");
-    strCmd += QString(" -video_size %1x%2").arg(camSize.width()).arg(camSize.height());
+    strCmd += QString(" -video_size %1x%2").arg(exportSize.width()).arg(exportSize.height());
     strCmd += QString(" -framerate %1").arg(mDesc.fps);
 
     //strCmd += QString( " -r %1").arg( exportFps );
@@ -444,8 +335,7 @@ Status MovieExporter::generateMovie(
         strCmd += QString(" -i \"%1\" ").arg(tempAudioPath);
     }
 
-    strCmd += QString(" -s %1x%2").arg(exportSize.width()).arg(exportSize.height());
-    if(strOutputFile.endsWith(".apng"))
+    if (strOutputFile.endsWith(".apng", Qt::CaseInsensitive))
     {
         strCmd += QString(" -plays %1").arg(loop ? "0" : "1");
     }
@@ -454,6 +344,12 @@ Status MovieExporter::generateMovie(
     {
         strCmd += QString(" -pix_fmt yuv420p");
     }
+
+    if (strOutputFile.endsWith(".avi", Qt::CaseInsensitive))
+    {
+        strCmd += " -q:v 5";
+    }
+
     strCmd += " -y";
     strCmd += QString(" \"%1\"").arg(strOutputFile);
 
@@ -477,10 +373,12 @@ Status MovieExporter::generateMovie(
             QImage imageToExport = imageToExportBase.copy();
             QPainter painter(&imageToExport);
 
+            QTransform view = cameraLayer->getViewAtFrame(currentFrame);
             painter.setWorldTransform(view * centralizeCamera);
             painter.setWindow(QRect(0, 0, camSize.width(), camSize.height()));
 
             obj->paintImage(painter, currentFrame, false, true);
+            painter.end();
 
             // Should use sizeInBytes instead of byteCount to support large images,
             // but this is only supported in QT 5.10+
@@ -552,8 +450,6 @@ Status MovieExporter::generateGif(
     }
     imageToExportBase.fill(bgColor);
 
-    QTransform view = cameraLayer->getViewAtFrame(currentFrame);
-
     QSize camSize = cameraLayer->getViewSize();
     QTransform centralizeCamera;
     centralizeCamera.translate(camSize.width() / 2, camSize.height() / 2);
@@ -562,15 +458,13 @@ Status MovieExporter::generateGif(
 
     QString strCmd = QString("\"%1\"").arg(ffmpegPath);
     strCmd += QString(" -f rawvideo -pixel_format bgra");
-    strCmd += QString(" -video_size %1x%2").arg(camSize.width()).arg(camSize.height());
+    strCmd += QString(" -video_size %1x%2").arg(exportSize.width()).arg(exportSize.height());
     strCmd += QString(" -framerate %1").arg(mDesc.fps);
 
     strCmd += " -i -";
 
     strCmd += " -y";
 
-    strCmd += QString(" -s %1x%2").arg(exportSize.width()).arg(exportSize.height());
-
     strCmd += " -filter_complex \"[0:v]palettegen [p]; [0:v][p] paletteuse\"";
 
     strCmd += QString(" -loop %1").arg(loop ? "0" : "-1");
@@ -598,6 +492,7 @@ Status MovieExporter::generateGif(
         QImage imageToExport = imageToExportBase.copy();
         QPainter painter(&imageToExport);
 
+        QTransform view = cameraLayer->getViewAtFrame(currentFrame);
         painter.setWorldTransform(view * centralizeCamera);
         painter.setWindow(QRect(0, 0, camSize.width(), camSize.height()));
 
